<button class="button-full-screen button button-icon" type="button" data-betty-full-screen-target="{% raw %}{{{ betty-maps-control-full-screen-target }}}{% endraw %}">{% trans %}Toggle full screen{% endtrans %}</button>
