$form-check-input-border: $border-width solid $form-element-border-color;
$form-check-input-border-radius: 0;
$form-check-input-checked-color: $black;
$form-check-input-checked-bg-color: white;
$form-check-input-checked-border-color: $form-element-border-color;
$form-check-input-width: 1.5em;
