{% extends 'base.html.j2' %}
{% set page_title = page_title | default(entity_type.plugin_label_plural() | localize) %}
{% do breadcrumbs.append(
    page_title,
    page_resource | url(absolute=true)
) %}
{% block page_summary %}
    <p>
        {{ entity_type.plugin_label_count(entities | length) | localize | html_lang }}
    </p>
{% endblock %}
