{% if citer | length > 0 %}
    <div class="background-embellished mt-5 py-5" data-bs-theme="light-secondary">
        <div class="container">
            <section id="references" class="section">
                <h2 class="section-heading">
                    {% trans %}References{% endtrans %}
                    {% with url = page_resource | url ~ '#references' %}
                        {% include 'component/permalink.html.j2' %}
                    {% endwith %}
                </h2>
                <ol>
                    {% for number, entity in citer %}
                        <li id="reference-{{ number }}" class="reference">
                            {% include 'entity/label--citation.html.j2' %}
                        </li>
                    {% endfor %}
                </ol>
            </section>
        </div>
    </div>
{% endif %}
