{%- set embedded = embedded | default(False) -%}
{%- set person_context = entity_contexts['person'] -%}
{%- set label_as_link = not embedded and person_context != entity.person and entity.person is persistent_entity_id and entity.person is public -%}
{%- if label_as_link -%}
    <a href="{{ entity.person | url }}">
{%- endif -%}
{% include 'entity/person-name-label.html.j2' %}
{%- if label_as_link -%}
    </a>
{%- endif -%}
{%- if entity is public and not embedded -%}
    {%- with citations = entity.citations -%}
        {%- include 'component/reference.html.j2' -%}
    {% endwith %}
{%- endif -%}
