.tree-section {
  /* Trees are full width, so leave enough vertical space for scrolling. */
  height: 50vh;
}

.tree {
  background-color: $white;
  border-color: var(--betty-border-color);
  border-style: solid;
  border-width: $border-width 0 $border-width 0;
  position: relative;

  &:fullscreen {
    border-width: 0;
  }

  .btn-full-screen {
    position: absolute;
    right: $spacer * 0.5;
    top: $spacer * 0.5;
    z-index: 1;
  }
}
