<p>
    {{ project.copyright_notice.summary | localize | html_lang }}
</p>
<p>
    {{ project.license.summary | localize | html_lang }}
</p>
<p>{% trans %}A family history as told by <a href="https://betty.readthedocs.io">Betty👵</a>{% endtrans %}</p>
{% for link in secondary_navigation_links %}
    <p><a href="{{ link.url | url }}">{{ link.label | localize | html_lang }}</a></p>
{% endfor %}
{% if job_context is defined %}
    <p>{% trans last_modified = localizer.format_datetime_datetime(job_context.start) %}Last modified on {{ last_modified }}{% endtrans %}</p>
{% endif %}
