{% with attributes = new_attributes(html_class=['overlay-background-contrast']),
    target = '{{{ betty-maps-control-full-screen-target }}}'
%}
    {% include 'component/button-full-screen.html.j2' %}
{% endwith %}
