@import "@fontsource/noto-sans/300.css";
@import "@fontsource/noto-sans/300-italic.css";
@import "@fontsource/noto-sans/400.css";
@import "@fontsource/noto-sans/400-italic.css";
@import "@fontsource/noto-sans/500.css";
@import "@fontsource/noto-sans/500-italic.css";
@import "@fontsource/noto-sans/600.css";
@import "@fontsource/noto-sans/600-italic.css";
@import "@fontsource/noto-sans/700.css";
@import "@fontsource/noto-sans/700-italic.css";
@import "@fontsource/noto-sans-arabic/300.css";
@import "@fontsource/noto-sans-arabic/400.css";
@import "@fontsource/noto-sans-arabic/500.css";
@import "@fontsource/noto-sans-arabic/600.css";
@import "@fontsource/noto-sans-arabic/700.css";
@import "@fontsource/noto-sans-hebrew/300.css";
@import "@fontsource/noto-sans-hebrew/400.css";
@import "@fontsource/noto-sans-hebrew/500.css";
@import "@fontsource/noto-sans-hebrew/600.css";
@import "@fontsource/noto-sans-hebrew/700.css";
