"""
Test utilities for :py:mod:`betty.ancestry.date`.
"""

from betty.ancestry.date import HasDate


class DummyHasDate(HasDate):
    """
    A dummy :py:class:`betty.ancestry.date.HasDate` implementation.
    """

    pass
