{% set attributes = attributes | default(new_attributes()) %}
{% do attributes.setdefault(html_id=generate_html_id()) %}
<div {% if form_control_attributes is defined %}{{ form_control_attributes }}{% endif %}>
    <input class="form-check-input" type="checkbox" value="{{ checkbox_value }}" {{ attributes }}>
    <label class="form-check-label" for="{{ attributes.html_id }}">{{ checkbox_label }}</label>
</div>
