"""
Test utilities for :py:mod:`betty.ancestry.locale`.
"""

from betty.ancestry.locale import HasLocale


class DummyHasLocale(HasLocale):
    """
    A dummy :py:class:`betty.ancestry.locale.HasLocale` implementation.
    """

    pass
