<svg xmlns="http://www.w3.org/2000/svg" width="49px" height="49px" viewBox="0 0 49 49">
    <circle cx="25" cy="25" r="15" fill="#94d668" stroke="#94d668" stroke-opacity="50%" stroke-width="10"/>
</svg>
