<div>
    <div class="map-selected-place-column">
        <button class="button-close button button-icon map-selected-place-close" type="button">{% trans %}Close{% endtrans %}</button>
        <div class="map-selected-place-content"></div>
    </div>
</div>
