{% extends 'base.html.j2' %}
{% set page_title = _('Page not found') %}
{% block page_content %}
    <p>{% trans %}I'm sorry, dear, but it seems this page does not exist.{% endtrans %}</p>
{% endblock %}
