{% set entities = entities | select('public') | list %}
{% if entities | length > 0 %}
    <ul class="entities">
        {% for entity in entities %}
            <li class="{{ loop.cycle('odd', 'even') }}">
                {% include ['entity/label--' + entity.plugin_id() + '.html.j2', 'entity/label.html.j2'] %}
                {% include ['entity/meta--' + entity.plugin_id() + '.html.j2', 'entity/meta.html.j2'] %}
            </li>
        {% endfor %}
    </ul>
{% endif %}
