<svg xmlns="http://www.w3.org/2000/svg" width="49px" height="49px" viewBox="0 0 49 49">
    <style>svg{filter: drop-shadow(0 0 5px #999);}</style>
    <circle cx="25" cy="25" r="19" fill="{{ project.extensions['raspberry-mint'].configuration.primary_color.hex }}"
            fill-opacity="50%"/>
    <circle cx="25" cy="25" r="4" fill="{{ project.extensions['raspberry-mint'].configuration.primary_color.hex }}"/>
</svg>
