<fieldset>
    <legend{% if checkboxes_label_visually_hidden %} class="visually-hidden"{% endif %}>{{ checkboxes_label }}</legend>
    {% for checkbox in checkboxes %}
        {% with checkbox_label = checkbox['label'],
            checkbox_value = checkbox['value'],
            attributes=checkbox['attributes'],
            form_control_attributes=checkbox['form_control_attributes']
        %}
            {% include 'component/checkbox.html.j2' %}
        {% endwith %}
    {% endfor %}
</fieldset>