@use "sass:map";
@import "bootstrap/scss/mixins/breakpoints";

@mixin -modal-fullscreen() {
  --bs-modal-header-padding: #{$container-padding-x * 0.5};
  --bs-modal-padding: #{$container-padding-x * 0.5};
}

@each $breakpoint in map.keys($grid-breakpoints) {
  $infix: breakpoint-infix($breakpoint, $grid-breakpoints);
  $postfix: if($infix != "", $infix + "-down", "");

  @include media-breakpoint-down($breakpoint) {
    .modal-fullscreen#{$postfix} {
      @include -modal-fullscreen;
    }
  }
}
