.image {
  background-color: var(--betty-background-contrast-color);
  border: $border-width solid var(--betty-border-color);

  // Ensure vector images such as SVGs are not stretched.
  object-fit: cover;
  width: 100%;
}

.image-link:focus .image {
  border-color: var(--betty-focus-color);
}
