<div class="overlay-background-contrast py-2" data-bs-theme="dark-secondary">
    <div class="map-selected-place-column">
        <div class="d-flex flex-row-reverse gap-2">
            {% with button_label = _('Close'),
                attributes=new_attributes(html_class=['btn-icon', 'map-selected-place-close', 'mb-2'])
            %}
                {% include 'component/button.html.j2' %}
            {% endwith %}
            <div class="map-selected-place-content flex-grow-1"></div>
        </div>
    </div>
</div>
