{% with attributes = attributes | default(new_attributes()) %}
    {% do attributes.html_class.append('btn-full-screen') %}
    {% do attributes.html_class.append('btn-icon') %}
    {% do attributes.set_data(betty_full_screen_target=target) %}
    {% set button_label = _('Toggle full screen') %}
    {% include 'component/button.html.j2' %}
{% endwith %}
