<!doctype html>
<html>
<head>
    <title>{{ project.configuration.title | localize }}</title>
    <meta http-equiv="refresh" content="0; url={{ 'betty:///index.html' | url(locale=project.configuration.locales.default.locale) }}">
</head>
<body>
</body>
</html>
