$primary: {{ project.extensions['raspberry-mint'].configuration.primary_color.hex }};
$secondary: {{ project.extensions['raspberry-mint'].configuration.secondary_color.hex }};
$tertiary: {{ project.extensions['raspberry-mint'].configuration.tertiary_color.hex }};
