.entities {
  margin: 0;
  padding: 0;
}

.entities li {
  list-style: none;
  margin-top: 0.5em;
  padding: 0.5em;
}

.entities li.even {
  background-color: #f7f7f7;
}

.featured-entities {
  display: grid;
  grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
}

.featured-entity {
  height: 500px;
}

.featured-entity h2 {
  margin: 0;
}

.featured-entity-intro {
  height: calc(30% - 2rem);
  padding: 1rem;
  width: calc(100% - 2rem);
}

.featured-entity-feature {
  height: 70%;
  width: 100%;
}

.featured-entity-feature-link {
  display: block;
  height: 100%;
  width: 100%;
}

.featured-entity-feature img {
  height: 350px;
  object-fit: cover;
  width: 100%;
}

@media (width >= 900px) {
  .featured-entities {
    grid-template-columns: repeat(auto-fit, minmax(450px, 1fr));
  }

  .featured-entity {
    height: 750px;
  }

  .featured-entity-intro {
    height: calc(20% - 2rem);
    padding: 1rem 3rem;
    width: calc(100% - 6rem);
  }

  .featured-entity-feature {
    height: 80%;
  }

  .featured-entity-feature img {
    height: 600px;
  }
}
