{% set label = _('Permanent link to this section.') %}
<sup>
    <a href="{{ url }}" title="{{ label }}"
       class="permalink">
        <span class="permalink-wrapper">
            <span class="visually-hidden">{{ label }}</span>
        </span>
    </a>
</sup>
