<!doctype html>
<html lang="{{ localizer.locale }}">
<head>
    {% include 'head.html.j2' %}
</head>
<body>
<div id="content">
    {% if page_title is defined %}
        <h1>{{ page_title }}</h1>
    {% endif %}
    {% block page_content %}{% endblock %}
</div>
{% include 'linked-data.html.j2' %}
{% if 'webpack' in project.extensions %}
    {% include 'webpack-entry-loader.html.j2' %}
{% endif %}
{% include 'scripts.html.j2' %}
</body>
</html>
