.citation {
  font-size: 0.8rem;
  vertical-align: super;
}

#references {
  font-size: 0.9rem;
  margin-top: 3rem;
}

.reference {
  padding: 0.3rem;
}

.reference:target {
  background-color: #eee;
}

.citation-author::after {
  content: ". ";
}

.citation-title {
  font-style: italic;
}

.citation-title::before {
  content: "\"";
}

.citation-title::after {
  content: "\". ";
}

.citation-publisher::after {
  content: ", ";
}

.citation-location::after {
  content: ". ";
}

.citation-date::after {
  content: ". ";
}
